#include "DEV_Config.h"
#include "GUI_Paint.h"
#include "./lib/e-Paper/EPD_2in66b.h"
#include <stdlib.h> // malloc() free()
#include <stdio.h>  // printf()

// Image Buffers
unsigned char gImage_b[5630];
unsigned char gImage_r[5630];

int main(void)
{
    // GPIO setup
    stdio_init_all();
    gpio_init(25);
    gpio_set_dir(25, GPIO_OUT);
    gpio_put(25, 0);

    // Initialize EPD and allocate memory for image buffers
    if (DEV_Module_Init() != 0)
    {
        return -1;
    }
    EPD_2IN66B_Init();
    EPD_2IN66B_Clear();

    // Create new image cache for black and red images
    UBYTE *BlackImage, *RedImage;
    UWORD Imagesize = ((EPD_2IN66B_WIDTH % 8 == 0) ? (EPD_2IN66B_WIDTH / 8) : (EPD_2IN66B_WIDTH / 8 + 1)) * EPD_2IN66B_HEIGHT;
    if ((BlackImage = (UBYTE *)malloc(Imagesize)) == NULL || (RedImage = (UBYTE *)malloc(Imagesize)) == NULL)
    {
        printf("Failed to apply for memory...\r\n");
        return -1;
    }

    Paint_NewImage(BlackImage, EPD_2IN66B_WIDTH, EPD_2IN66B_HEIGHT, 270, WHITE);
    Paint_Clear(WHITE);
    Paint_NewImage(RedImage, EPD_2IN66B_WIDTH, EPD_2IN66B_HEIGHT, 270, WHITE);
    Paint_Clear(WHITE);

    EPD_2IN66B_Sleep();
    DEV_Delay_ms(2000); // Important delay, at least 2s

    // Main loop
    int i = 0;
    char color = 'b'; // Variable to track current color buffer (black or red)

    while (1)
    {
        // Toggle GPIO pin to indicate activity
        gpio_put(25, !gpio_get(25));

        // Read incoming data
        int c = getchar_timeout_us(500000);

        // If there's incoming data and buffer index is within limits
        if (i < 5630 && c != PICO_ERROR_TIMEOUT)
        {
            // Store data in appropriate buffer (black or red)
            if (color == 'b')
            {
                gImage_b[i] = c;
            }
            else if (color == 'r')
            {
                gImage_r[i] = c;
            }
            i++;

            // If black image buffer is filled, switch to red buffer
            if (i == 5630 && color == 'b')
            {
                color = 'r';
                i = 0;
            }
            // If red image buffer is filled, proceed to display
            else if (i == 5630 && color == 'r')
            {
                printf("Printing to screen!\r\n");

                // Initialize EPD and clear screen (best practice)
                EPD_2IN66B_Init();
                // EPD_2IN66B_Clear();

                DEV_Delay_ms(200);

                // Display black and red images
                Paint_SelectImage(BlackImage);
                Paint_Clear(WHITE);
                Paint_DrawBitMap(gImage_b);

                Paint_SelectImage(RedImage);
                Paint_Clear(WHITE);
                Paint_DrawBitMap(gImage_r);

                EPD_2IN66B_Display(BlackImage, RedImage);
                DEV_Delay_ms(3000);

                // Enter sleep mode
                EPD_2IN66B_Sleep();

                // Reset color and index for next cycle
                color = 'b';
                i = 0;
            }
        }
        else
        {
            // Reset color and index on serial timeout
            color = 'b';
            i = 0;
        }
    }

    return 0;
}
